"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.Item = void 0;
var _extends3 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));
var _react = _interopRequireWildcard(require("react"));
var _classnames = _interopRequireDefault(require("classnames"));
var _ItemTrackerProvider = require("./ItemTrackerProvider");
var _constants = require("../constants");
var _utils = require("./utils");
var _core = require("../core");
var _excluded = ["id", "children", "className", "style", "triggerEvent", "data", "propsFromTrigger", "keyMatcher", "onClick", "disabled", "hidden", "closeOnClick", "handlerEvent"];
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { "default": e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n["default"] = e, t && t.set(e, n), n; }
var Item = exports.Item = function Item(_ref) {
  var _extends2, _cx;
  var id = _ref.id,
    children = _ref.children,
    className = _ref.className,
    style = _ref.style,
    triggerEvent = _ref.triggerEvent,
    data = _ref.data,
    propsFromTrigger = _ref.propsFromTrigger,
    keyMatcher = _ref.keyMatcher,
    _ref$onClick = _ref.onClick,
    onClick = _ref$onClick === void 0 ? _constants.NOOP : _ref$onClick,
    _ref$disabled = _ref.disabled,
    disabled = _ref$disabled === void 0 ? false : _ref$disabled,
    _ref$hidden = _ref.hidden,
    hidden = _ref$hidden === void 0 ? false : _ref$hidden,
    _ref$closeOnClick = _ref.closeOnClick,
    closeOnClick = _ref$closeOnClick === void 0 ? true : _ref$closeOnClick,
    _ref$handlerEvent = _ref.handlerEvent,
    handlerEvent = _ref$handlerEvent === void 0 ? 'onClick' : _ref$handlerEvent,
    rest = (0, _objectWithoutPropertiesLoose2["default"])(_ref, _excluded);
  var itemNode = (0, _react.useRef)();
  var itemTracker = (0, _ItemTrackerProvider.useItemTrackerContext)();
  var handlerParams = {
    id: id,
    data: data,
    triggerEvent: triggerEvent,
    props: propsFromTrigger
  };
  var isDisabled = (0, _utils.getPredicateValue)(disabled, handlerParams);
  var isHidden = (0, _utils.getPredicateValue)(hidden, handlerParams);
  function handleClick(e) {
    handlerParams.event = e;
    e.stopPropagation();
    if (!isDisabled) {
      !closeOnClick ? onClick(handlerParams) : dispatchUserHanlder();
    }
  }

  // provide a feedback to the user that the item has been clicked before closing the menu
  function dispatchUserHanlder() {
    var node = itemNode.current;
    node.focus();
    node.addEventListener('animationend',
    // defer, required for react 17
    function () {
      return setTimeout(_core.contextMenu.hideAll);
    }, {
      once: true
    });
    node.classList.add(_constants.CssClass.itemClickedFeedback);
    onClick(handlerParams);
  }
  function registerItem(node) {
    if (node && !isDisabled) {
      itemNode.current = node;
      itemTracker.set(node, {
        node: node,
        isSubmenu: false,
        keyMatcher: !isDisabled && (0, _utils.isFn)(keyMatcher) && function (e) {
          if (keyMatcher(e)) {
            e.stopPropagation();
            e.preventDefault();
            handlerParams.event = e;
            dispatchUserHanlder();
          }
        }
      });
    }
  }
  function handleKeyDown(e) {
    if (e.key === 'Enter' || e.key === ' ') {
      e.stopPropagation();
      handlerParams.event = e;
      dispatchUserHanlder();
    }
  }
  if (isHidden) return null;
  return /*#__PURE__*/_react["default"].createElement("div", (0, _extends3["default"])({}, (0, _extends3["default"])({}, rest, (_extends2 = {}, _extends2[handlerEvent] = handleClick, _extends2)), {
    className: (0, _classnames["default"])(_constants.CssClass.item, className, (_cx = {}, _cx["" + _constants.CssClass.itemDisabled] = isDisabled, _cx)),
    style: style,
    onKeyDown: handleKeyDown,
    ref: registerItem,
    tabIndex: -1,
    role: "menuitem",
    "aria-disabled": isDisabled
  }), /*#__PURE__*/_react["default"].createElement("div", {
    className: _constants.CssClass.itemContent
  }, children));
};