import React from 'react';
import { MenuId, MenuAnimation, Theme } from '../types';
export interface MenuProps extends Omit<React.HTMLAttributes<HTMLElement>, 'id'> {
    /**
     * Unique id to identify the menu. Use to Trigger the corresponding menu
     */
    id: MenuId;
    /**
     * Theme is appended to `contexify_theme-${given theme}`.
     *
     * Built-in theme are `light` and `dark`
     */
    theme?: Theme;
    /**
     * Animation is appended to
     * - `.contexify_willEnter-${given animation}`
     * - `.contexify_willLeave-${given animation}`
     *
     * - To disable all animations you can pass `false`
     * - To disable only the enter or the exit animation you can provide an object `{enter: false, exit: 'exitAnimation'}`
     *
     * - default is set to `fade`
     */
    animation?: MenuAnimation;
    /**
     * Disables menu repositioning if outside screen.
     * This may be neeeded in some cases when using custom position.
     */
    disableBoundariesCheck?: boolean;
    /**
     * Prevents scrolling the window on when typing. Defaults to true.
     */
    preventDefaultOnKeydown?: boolean;
    /**
     * Used to track menu visibility
     */
    onVisibilityChange?: (isVisible: boolean) => void;
}
export declare const Menu: ({ id, theme, style, className, animation, preventDefaultOnKeydown, disableBoundariesCheck, onVisibilityChange, ...rest }: MenuProps) => JSX.Element;
